require 'thor'
require 'logger'

module Clockker
  class CLI < Thor
    hostname = `hostname`.strip
    macaddrpart = `ifconfig` =~ /en0.*?(([A-F0-9]{2}:){5}[A-F0-9]{2})/im
    macaddrpart = $1[2..4]
    identifier = hostname+macaddrpart

    desc "watch", "Start the Clockker watcher"
    method_option :token, aliases: '-t', type: :string, desc: "REQUIRED Token for communicating to Clockk server, provided by Clockk"
    method_option :region, aliases: '-r', type: :string, desc: "REQUIRED server region. e.g. ca-central, us-east, eu-frankfurt"
    method_option :whitelist, aliases: '-w', type: :array, desc: "Space-separated list of absolute paths to watch. Defaults to your home directory (~)."
    method_option :blacklist, aliases: '-b', type: :array, desc: "Space-separated list of absolute paths to ignore that are undeneath your whitelisted paths. Defaults to ~/Library."
    method_option :identifier, aliases: '-i', type: :string, desc: "The identifier for this computer.", default: identifier
    method_option :submit_frequency, aliases: '-s', default: 300, desc: "Submit changeset to Clockk server every N seconds", type: :numeric
    method_option :verbose, aliases: '-v', default: 'fatal', lazy_default: 'info', enum: %w{debug info warn error fatal}, desc: "Set verbosity (default: info)"
    method_option :development, aliases: '-d', default: false, desc: "Submit to a development server at localhost:4000 as well as the production server"
    def watch
      logger = Logger.new(STDERR)
      case options[:verbose]
      when 'debug'
        logger.level = Logger::DEBUG
      when 'info'
        logger.level = Logger::INFO
      when 'warn'
        logger.level = Logger::WARN
      when 'error'
        logger.level = Logger::ERROR
      else # fatal and no preference
        logger.level = Logger::FATAL
      end
      logger.debug "Setting verbosity to #{options[:verbose]}"
      
      # config_params = {}
      # config_params[:submit_frequency] = options[:submit_frequency].to_i if options[:submit_frequency].present?
      # config_params[:log_level] = options[:verbose] if options[:verbose].present?
      # config_params[:token] = options[:token] if options[:token].present?
      # config_params[:whitelist] = options[:whitelist] if options[:whitelist].present?
      # config_params[:blacklist] = options[:blacklist] if options[:blacklist].present?

      clockker_config = Clockker::Config.new(
        submit_frequency: options[:submit_frequency].to_i,
        log_level: options[:verbose],
        identifier: options[:identifier]
      )

      clockker_config.token = options[:token] if options[:token]
      clockker_config.region = options[:region] if options[:region]
      clockker_config.whitelist = options[:whitelist] if options[:whitelist]
      clockker_config.blacklist = options[:blacklist] if options[:blacklist]
      clockker_config.development = options[:development] if options[:development]

      if options[:token].nil? && clockker_config.token.nil?
        puts "You must provide a value for the token, either as a parameter (--token=x) or in the ~/.clockker_config file."
        exit
      end

      if options[:region].nil? && clockker_config.region.nil?
        puts "You must provide a value for the server region (e.g. ca-central, us-east, eu-frankfurt, etc.), either as a parameter (--region=x) or in the ~/.clockker_config file."
        exit
      end

      Clockker::Watcher.new(clockker_config, logger: logger)
    end

    desc "version", "Show Clockker version"
    def version
      puts Clockker.version_string
    end
  end
end